/**
 * Created by PanJiaChen on 16/11/18.
 */

/**
 * Parse the time to string
 * @param {(Object|string|number)} time
 * @returns {string | null}
 */
export function parseDate(time) {
  return parseTime(time, '{y}-{m}-{d}')
}

/**
 * Parse the time to string
 * @param {(Object|string|number)} time
 * @param {string} cFormat
 * @returns {string | null}
 */
export function parseTime(time, cFormat) {
  if (arguments.length === 0 || !time) {
    return null
  }
  const format = cFormat || '{y}-{m}-{d} {h}:{i}:{s}'
  let date
  if (typeof time === 'object') {
    date = time
  } else {
    if ((typeof time === 'string')) {
      if ((/^[0-9]+$/.test(time))) {
        // support "1548221490638"
        time = parseInt(time)
      } else {
        // support safari
        // https://stackoverflow.com/questions/4310953/invalid-date-in-safari
        time = time.replace(new RegExp(/-/gm), '/')
      }
    }

    if ((typeof time === 'number') && (time.toString().length === 10)) {
      time = time * 1000
    }
    date = new Date(time)
  }
  const formatObj = {
    y: date.getFullYear(),
    m: date.getMonth() + 1,
    d: date.getDate(),
    h: date.getHours(),
    i: date.getMinutes(),
    s: date.getSeconds(),
    a: date.getDay()
  }
  const time_str = format.replace(/{([ymdhisa])+}/g, (result, key) => {
    const value = formatObj[key]
    // Note: getDay() returns 0 on Sunday
    if (key === 'a') { return ['日', '一', '二', '三', '四', '五', '六'][value ] }
    return value.toString().padStart(2, '0')
  })
  return time_str
}

/**
 * @param {number} time
 * @param {string} option
 * @returns {string}
 */
export function formatTime(time, option) {
  if (('' + time).length === 10) {
    time = parseInt(time) * 1000
  } else {
    time = +time
  }
  const d = new Date(time)
  const now = Date.now()

  const diff = (now - d) / 1000

  if (diff < 30) {
    return '刚刚'
  } else if (diff < 3600) {
    // less 1 hour
    return Math.ceil(diff / 60) + '分钟前'
  } else if (diff < 3600 * 24) {
    return Math.ceil(diff / 3600) + '小时前'
  } else if (diff < 3600 * 24 * 2) {
    return '1天前'
  }
  if (option) {
    return parseTime(time, option)
  } else {
    return (
      d.getMonth() +
      1 +
      '月' +
      d.getDate() +
      '日' +
      d.getHours() +
      '时' +
      d.getMinutes() +
      '分'
    )
  }
}

/**
 * @param {string} url
 * @returns {Object}
 */
export function getQueryObject(url) {
  url = url == null ? window.location.href : url
  const search = url.substring(url.lastIndexOf('?') + 1)
  const obj = {}
  const reg = /([^?&=]+)=([^?&=]*)/g
  search.replace(reg, (rs, $1, $2) => {
    const name = decodeURIComponent($1)
    let val = decodeURIComponent($2)
    val = String(val)
    obj[name] = val
    return rs
  })
  return obj
}

/**
 * @param {string} input value
 * @returns {number} output value
 */
export function byteLength(str) {
  // returns the byte length of an utf8 string
  let s = str.length
  for (var i = str.length - 1; i >= 0; i--) {
    const code = str.charCodeAt(i)
    if (code > 0x7f && code <= 0x7ff) s++
    else if (code > 0x7ff && code <= 0xffff) s += 2
    if (code >= 0xDC00 && code <= 0xDFFF) i--
  }
  return s
}

/**
 * @param {Array} actual
 * @returns {Array}
 */
export function cleanArray(actual) {
  const newArray = []
  for (let i = 0; i < actual.length; i++) {
    if (actual[i]) {
      newArray.push(actual[i])
    }
  }
  return newArray
}

/**
 * @param {Object} json
 * @returns {Array}
 */
export function param(json) {
  if (!json) return ''
  return cleanArray(
    Object.keys(json).map(key => {
      if (json[key] === undefined) return ''
      return encodeURIComponent(key) + '=' + encodeURIComponent(json[key])
    })
  ).join('&')
}

/**
 * @param {string} url
 * @returns {Object}
 */
export function param2Obj(url) {
  const search = decodeURIComponent(url.split('?')[1]).replace(/\+/g, ' ')
  if (!search) {
    return {}
  }
  const obj = {}
  const searchArr = search.split('&')
  searchArr.forEach(v => {
    const index = v.indexOf('=')
    if (index !== -1) {
      const name = v.substring(0, index)
      const val = v.substring(index + 1, v.length)
      obj[name] = val
    }
  })
  return obj
}

/**
 * @param {string} val
 * @returns {string}
 */
export function html2Text(val) {
  const div = document.createElement('div')
  div.innerHTML = val
  return div.textContent || div.innerText
}

/**
 * Merges two objects, giving the last one precedence
 * @param {Object} target
 * @param {(Object|Array)} source
 * @returns {Object}
 */
export function objectMerge(target, source) {
  if (typeof target !== 'object') {
    target = {}
  }
  if (Array.isArray(source)) {
    return source.slice()
  }
  Object.keys(source).forEach(property => {
    const sourceProperty = source[property]
    if (typeof sourceProperty === 'object') {
      target[property] = objectMerge(target[property], sourceProperty)
    } else {
      target[property] = sourceProperty
    }
  })
  return target
}

/**
 * @param {HTMLElement} element
 * @param {string} className
 */
export function toggleClass(element, className) {
  if (!element || !className) {
    return
  }
  let classString = element.className
  const nameIndex = classString.indexOf(className)
  if (nameIndex === -1) {
    classString += '' + className
  } else {
    classString =
      classString.substr(0, nameIndex) +
      classString.substr(nameIndex + className.length)
  }
  element.className = classString
}

/**
 * @param {string} type
 * @returns {Date}
 */
export function getTime(type) {
  if (type === 'start') {
    return new Date().getTime() - 3600 * 1000 * 24 * 90
  } else {
    return new Date(new Date().toDateString())
  }
}

/**
 * @param {Function} func
 * @param {number} wait
 * @param {boolean} immediate
 * @return {*}
 */
export function debounce(func, wait, immediate) {
  let timeout, args, context, timestamp, result

  const later = function() {
    // 据上一次触发时间间隔
    const last = +new Date() - timestamp

    // 上次被包装函数被调用时间间隔 last 小于设定时间间隔 wait
    if (last < wait && last > 0) {
      timeout = setTimeout(later, wait - last)
    } else {
      timeout = null
      // 如果设定为immediate===true，因为开始边界已经调用过了此处无需调用
      if (!immediate) {
        result = func.apply(context, args)
        if (!timeout) context = args = null
      }
    }
  }

  return function(...args) {
    context = this
    timestamp = +new Date()
    const callNow = immediate && !timeout
    // 如果延时不存在，重新设定延时
    if (!timeout) timeout = setTimeout(later, wait)
    if (callNow) {
      result = func.apply(context, args)
      context = args = null
    }

    return result
  }
}

/**
 * This is just a simple version of deep copy
 * Has a lot of edge cases bug
 * If you want to use a perfect deep copy, use lodash's _.cloneDeep
 * @param {Object} source
 * @returns {Object}
 */
export function deepClone(source) {
  if (!source && typeof source !== 'object') {
    throw new Error('error arguments', 'deepClone')
  }
  const targetObj = source.constructor === Array ? [] : {}
  Object.keys(source).forEach(keys => {
    if (source[keys] && typeof source[keys] === 'object') {
      targetObj[keys] = deepClone(source[keys])
    } else {
      targetObj[keys] = source[keys]
    }
  })
  return targetObj
}

/**
 * @param {Array} arr
 * @returns {Array}
 */
export function uniqueArr(arr) {
  return Array.from(new Set(arr))
}

/**
 * @returns {string}
 */
export function createUniqueString() {
  const timestamp = +new Date() + ''
  const randomNum = parseInt((1 + Math.random()) * 65536) + ''
  return (+(randomNum + timestamp)).toString(32)
}

/**
 * Check if an element has a class
 * @param ele
 * @param {string} cls
 * @returns {boolean}
 */
export function hasClass(ele, cls) {
  return !!ele.className.match(new RegExp('(\\s|^)' + cls + '(\\s|$)'))
}

/**
 * Add class to element
 * @param ele
 * @param {string} cls
 */
export function addClass(ele, cls) {
  if (!hasClass(ele, cls)) ele.className += ' ' + cls
}

/**
 * Remove class from element
 * @param ele
 * @param {string} cls
 */
export function removeClass(ele, cls) {
  if (hasClass(ele, cls)) {
    const reg = new RegExp('(\\s|^)' + cls + '(\\s|$)')
    ele.className = ele.className.replace(reg, ' ')
  }
}

export function strArr2NumArr(actual) {
  const newArray = []
  for (let i = 0; i < actual.length; i++) {
    if (actual[i]) {
      newArray.push(parseInt(actual[i]))
    }
  }
  return newArray
}

// 格式化日期：yyyy-MM-dd
export function formatDate(date) {
  const myyear = date.getFullYear()
  let mymonth = date.getMonth() + 1
  let myweekday = date.getDate()
  if (mymonth < 10) {
    mymonth = '0' + mymonth
  }
  if (myweekday < 10) {
    myweekday = '0' + myweekday
  }
  return (myyear + '-' + mymonth + '-' + myweekday)
}

export function formatDateByYmd(y, m, d) {
  let dateStr = ''
  if (m < 10) {
    dateStr = y + '-0' + m
  } else {
    dateStr = y + '-' + m
  }
  if (d < 10) {
    dateStr += '-0' + d
  } else {
    dateStr += '-' + d
  }
  return dateStr
}

// 获得某月的天数
export function getMonthDays(myMonth) {
  const now = new Date() // 当前日期
  const nowYear = now.getFullYear() // 当前年
  const monthStartDate = new Date(nowYear, myMonth, 1)
  const monthEndDate = new Date(nowYear, myMonth + 1, 1)
  return (monthEndDate - monthStartDate) / (1000 * 60 * 60 * 24)
}

// 获得本季度的开始月份
export function getQuarterStartMonth() {
  const now = new Date() // 当前日期
  const nowMonth = now.getMonth() // 当前月
  let quarterStartMonth = 0
  if (nowMonth < 3) {
    quarterStartMonth = 0
  }
  if (nowMonth > 2 && nowMonth < 6) {
    quarterStartMonth = 3
  }
  if (nowMonth > 5 && nowMonth < 9) {
    quarterStartMonth = 6
  }
  if (nowMonth > 8) {
    quarterStartMonth = 9
  }
  return quarterStartMonth
}

//  获得本周的开始日期
export function getWeekStartDate() {
  const now = new Date() // 当前日期
  const nowDay = now.getDate() // 当前日
  const nowDayOfWeek = now.getDay() - 1 // 今天本周的第几天
  const nowMonth = now.getMonth()
  const nowYear = now.getFullYear() // 当前年
  const weekStartDate = new Date(nowYear, nowMonth, nowDay - nowDayOfWeek)
  return formatDate(weekStartDate)
}

//  获得本周的结束日期
export function getWeekEndDate() {
  const now = new Date() // 当前日期
  const nowDay = now.getDate() // 当前日
  const nowMonth = now.getMonth() // 当前月
  const nowYear = now.getFullYear() // 当前年
  const nowDayOfWeek = now.getDay() - 1 // 今天本周的第几天
  const weekEndDate = new Date(nowYear, nowMonth, nowDay + (6 - nowDayOfWeek))
  return formatDate(weekEndDate)
}

//  获得上周的开始日期
export function getLastWeekStartDate() {
  const now = new Date() // 当前日期
  const nowDay = now.getDate() // 当前日
  const nowMonth = now.getMonth() // 当前月
  const nowYear = now.getFullYear() // 当前年
  const nowDayOfWeek = now.getDay() - 1 // 今天本周的第几天
  const weekStartDate = new Date(nowYear, nowMonth, nowDay - nowDayOfWeek - 7)
  return formatDate(weekStartDate)
}

// 获得上周的结束日期
export function getLastWeekEndDate() {
  const now = new Date() // 当前日期
  const nowDay = now.getDate() // 当前日
  const nowMonth = now.getMonth() // 当前月
  const nowYear = now.getFullYear() // 当前年
  const nowDayOfWeek = now.getDay() - 1 // 今天本周的第几天
  const weekEndDate = new Date(nowYear, nowMonth, nowDay - nowDayOfWeek - 1)
  return formatDate(weekEndDate)
}

// 获得本月的开始日期
export function getMonthStartDate() {
  const now = new Date() // 当前日期
  const nowMonth = now.getMonth() // 当前月
  const nowYear = now.getFullYear() // 当前年
  const monthStartDate = new Date(nowYear, nowMonth, 1)
  return formatDate(monthStartDate)
}
// 获得本月的结束日期
export function getMonthEndDate() {
  const now = new Date() // 当前日期
  const nowMonth = now.getMonth() // 当前月
  const nowYear = now.getFullYear() // 当前年
  const monthEndDate = new Date(nowYear, nowMonth, getMonthDays(nowMonth))

  return formatDate(monthEndDate)
}

// 获得本年的开始日期
export function getYearStartDate() {
  const now = new Date() // 当前日期
  const nowYear = now.getFullYear() // 当前年
  const monthStartDate = new Date(nowYear, 1, 1)
  return formatDate(monthStartDate)
}
// 获得本年的结束日期
export function getYearEndDate() {
  const now = new Date() // 当前日期
  const nowYear = now.getFullYear() // 当前年
  const yarEndDate = new Date(nowYear, 11, 31)
  return formatDate(yarEndDate)
}

// 获得上月开始时间
export function getLastMonthStartDate() {
  const now = new Date() // 当前日期
  const nowYear = now.getFullYear() // 当前年
  const nowMonth = now.getMonth() // 当前月
  let lastMonthStartDate
  if (nowMonth === 0) {
    lastMonthStartDate = new Date(nowYear - 1, 12, 1)
  } else {
    lastMonthStartDate = new Date(nowYear, nowMonth - 1, 1)
  }
  return formatDate(lastMonthStartDate)
}
// 获得上月结束时间
export function getLastMonthEndDate() {
  const now = new Date() // 当前日期
  const nowMonth = now.getMonth() // 当前月
  const nowYear = now.getFullYear() // 当前年

  var lastMonthEndDate
  if (nowMonth === 0) {
    lastMonthEndDate = new Date(nowYear - 1, 12, 31)
  } else {
    lastMonthEndDate = new Date(nowYear, nowMonth - 1, getMonthDays(nowMonth - 1))
  }
  return formatDate(lastMonthEndDate)
}

// 获得本季度的开始日期
export function getQuarterStartDate() {
  const now = new Date() // 当前日期
  const nowYear = now.getFullYear() // 当前年
  const quarterStartDate = new Date(nowYear, getQuarterStartMonth(), 1)
  return formatDate(quarterStartDate)
}
// 或的本季度的结束日期
export function getQuarterEndDate() {
  const now = new Date() // 当前日期
  const nowYear = now.getFullYear() // 当前年
  const quarterEndMonth = getQuarterStartMonth() + 2
  const quarterStartDate = new Date(nowYear, quarterEndMonth, getMonthDays(quarterEndMonth))
  return formatDate(quarterStartDate)
}
// 平级转树结构
/* treeArr 基础数据
 * id 唯一id
 * parentId 父级id
 * childrenList 子级数组名*/
export function packageTreeData(params) {
  const treeArr = params && params.data || []
  const id = params && params.id || 'id'
  const parentId = params && params.parentId || 'parentId'
  const childrenList = params && params.children || 'children'
  const cloneData = JSON.parse(JSON.stringify(treeArr))
  return cloneData.filter(fatherItem => {
    const warpArr = cloneData.filter(sonItem => fatherItem[id] === sonItem[parentId])
    warpArr.length ? fatherItem[childrenList] = warpArr : null
    return !fatherItem[parentId]
  })
}
// 根据根id处理树结构
export function getNestedChildren(arr = [], parent = 0) {
  const res = []
  for (const item of arr) {
    if (item.parent_id === parent) {
      const children = getNestedChildren(arr, item.id)
      if (children.length) {
        item.children = children
      }
      res.push(item)
    }
  }
  return res
}
export function Uint8ArrayToString(fileData) {
  let dataString = ''
  for (let i = 0; i < fileData.length; i++) {
    dataString += String.fromCharCode(fileData[i])
  }

  return dataString
}
// 风险类型
export function riskType(val) {
  const strs = [
    '默认',
    '人',
    '物',
    '管'
  ]
  return strs[val]
}

// 风险等级
export function riskLevel(val) {
  const strs = [
    '默认',
    '重大',
    '较大',
    '一般',
    '较小'
  ]
  return strs[val]
}

// 风险等级
export function riskPointStatus(val) {
  const strs = [
    '未激活',
    '活动'
  ]
  if (val >= 0) return strs[val]
  else return '失控'
}

// 风险点等级
export function riskPointLevel(val) {
  const strs = [
    '默认',
    '重大',
    '较大',
    '一般',
    '较小'
  ]
  return strs[val]
}

export function taskPriority(i) {
  const strs = [
    '未知',
    '较低',
    '普通',
    '紧急',
    '非常紧急'
  ]
  return strs[i]
}

export function taskType(i) {
  const strs = [
    '未知',
    '常规',
    '审批任务',
    '巡检任务'
  ]
  return strs[i]
}

export function checkType(i) {
  const strs = [
    '未知',
    '常规检查',
    '临时检查',
    '扫码检查'
  ]
  return strs[i]
}

export function checklistType(i) {
  const strs = [
    '未知',
    '危险源清单',
    '专项清单',
    '评价清单'
  ]
  return strs[i]
}

export function taskStatus(i) {
  const strs = [
    '待处理',
    '已完成'
  ]
  if (i === -1) {
    return '已撤消'
  } else {
    return strs[i]
  }
}

export function checkStatus(i) {
  const strs = [
    '未开始',
    '执行中',
    '已完成'
  ]
  if (i === -1) {
    return '已撤消'
  } else {
    return strs[i]
  }
}

export function checkResult(i) {
  if (i >= 0) {
    const strs = [
      '未检查',
      '通过'
    ]
    return strs[i]
  } else {
    if (i === -1) {
      return '未通过'
    } else if (i === -2) {
      return '发现隐患'
    }
  }
}

export function targetType(val) {
  const strs = [
    '未知',
    '风险点',
    '科室',
    '设备设施',
    '人员',
    '医生'
  ]
  return strs[val]
}

export function checklistRecordStatus(i) {
  const strs = [
    '待处理',
    '未完成',
    '全部通过'
  ]
  if (i === -1) {
    return '没有通过'
  } else {
    return strs[i]
  }
}

export function dangerStatus(i) {
  if (i >= 0) {
    const strs = [
      '待提交',
      '待评审',
      '待整改',
      '待验收',
      '已整改'
    ]
    return strs[i]
  } else {
    return '已撤销'
  }
}

export function dangerLevel(i) {
  const strs = [
    '未知',
    '重大',
    '较大',
    '严重',
    '一般'
  ]
  return strs[i]
}

export function dangerSource(i) {
  const strs = [
    '自查',
    '内部反馈',
    '上级抽查',
    '政府执法'
  ]
  return strs[i]
}

export function rectifyCat(i) {
  const strs = [
    '自行整改',
    '外协整改'
  ]
  return strs[i]
}

export function wfInsStatus(i) {
  if (i >= 0) {
    const strs = [
      '未开始',
      '执行中',
      '已完成'
    ]
    return strs[i]
  } else {
    return '已撤销'
  }
}

export function wfActivityInsStatus(i) {
  const strs = [
    '未开始',
    '已完成'
  ]
  return strs[i]
}

export function dangerRectifyCat(i) {
  const strs = [
    '自行整改',
    '外协整改'
  ]
  return strs[i]
}

export function appType(i) {
  const strs = [
    '未知',
    '电脑端',
    '手机APP',
    '小程序'
  ]
  return strs[i]
}

export function alertLevel(i) {
  const strs = [
    '未知',
    '1级',
    '2级',
    '3级',
    '4级',
    '5级',
    '6级'
  ]
  return strs[i]
}

export function alertType(i) {
  const strs = [
    '提醒',
    '预警'
  ]
  return strs[i]
}

export function alertStatus(i) {
  if (i >= 0) {
    const strs = [
      '活动',
      '已解除'
    ]
    return strs[i]
  } else {
    return '已撤销'
  }
}

export function warningLevel(i) {
  const strs = [
    '未知',
    '1级',
    '2级',
    '3级',
    '4级',
    '5级',
    '6级'
  ]
  return strs[i]
}

export function accidentLevel(i) {
  const strs = [
    '未知',
    '特别重大事故',
    '重大事故',
    '较大事故',
    '一般事故'
  ]
  return strs[i]
}

export function warningStatus(i) {
  if (i >= 0) {
    const strs = [
      '待处理',
      '已处理'
    ]
    return strs[i]
  } else {
    return '已撤销'
  }
}

export function snapshotStatus(i) {
  if (i >= 0) {
    const strs = [
      '待处理',
      '已处理'
    ]
    return strs[i]
  } else {
    return '已撤销'
  }
}

export function ticketStatus(i) {
  if (i >= 0) {
    const strs = [
      '待处理',
      '已处理'
    ]
    return strs[i]
  } else {
    return '已撤销'
  }
}

export function hosStoryTitle(i) {
  if (i >= 0) {
    const strs = [
      '未知',
      '平安医院',
      '质量医院',
      '满意医院',
      '智慧医院',
      '健康医院',
      '节约型医院'
    ]
    return strs[i]
  } else {
    return '未知'
  }
}

export function hosStory(code) {
  const enums = {
    'pingAn': 1,
    'zhiLiang': 2,
    'manYi': 3,
    'zhiHui': 4,
    'jianKang': 5,
    'jieYue': 6
  }
  return enums[code]
}

export function riskLevelByCode(code) {
  const enums = {
    '红': 1,
    '橙': 2,
    '黄': 3,
    '蓝': 4
  }
  return enums[code]
}

export function ocAccess(i) {
  const strs = [
    '未接入',
    '已接入'
  ]
  return strs[i]
}

export function industryType(i) {
  const strs = [
    '未知',
    '行业',
    '领域'
  ]
  return strs[i]
}

export function isNotNull(value) {
  return value !== undefined && value !== 'undefined' && value !== null && value !== ''
}

export function generateClientId() {
  const len = 23
  const chars = 'ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678'
  const maxPos = chars.length
  let str = ''
  for (let i = 0; i < len; i++) {
    str += chars.charAt(Math.floor(Math.random() * maxPos))
  }
  return new Date().getTime() + str
}
